@@ -1,4 +1,6 @@
-# BSD 3-Clause License; see https://github.com/scikit-hep/awkward-1.0/blob/main/LICENSE
+# BSD 3-Clause License; see https://github.com/scikit-hep/awkward/blob/main/LICENSE
+
+from __future__ import annotations
 
 import threading
 
@@ -203,6 +205,12 @@ class DataSourceGenerator:
         else:
             cpp_code = None
 
+        init_method = (
+            "Initialize"
+            if hasattr(ROOT.RDF.RDataSource, "Initialize")
+            else "Initialise"
+        )
+
         if cpp_code is None:
             cpp_code = f"""
 namespace awkward {{
@@ -258,7 +266,7 @@ namespace awkward {{
             {cpp_code_resize_slots}
         }}
 
-        void Initialise() {{
+        void {init_method}() {{
             // initialize fEntryRanges
             const auto chunkSize = fSize / fNSlots;
             auto start = 0UL;
