@@ -1,4 +1,6 @@
-# BSD 3-Clause License; see https://github.com/scikit-hep/awkward-1.0/blob/main/LICENSE
+# BSD 3-Clause License; see https://github.com/scikit-hep/awkward/blob/main/LICENSE
+
+from __future__ import annotations
 
 import operator
 import sys
@@ -92,7 +94,7 @@ def test_box():
 
 def test_refcount():
     array = ak.highlevel.Array([[1.1, 2.2, 3.3], [], [4.4, 5.5]])
-    array.numba_type
+    array.numba_type  # noqa: B018 (needed to set refcount)
     assert [
         sys.getrefcount(x) == 2
         for x in (
@@ -469,7 +471,7 @@ def test_RecordView_refcount():
         ],
         check_valid=True,
     )[3]
-    record.numba_type
+    record.numba_type  # noqa: B018 (needed to set refcount)
     assert [
         sys.getrefcount(x) == 2
         for x in (
